@@ -45,7 +45,6 @@ sub new {
         libdir  => $params{libdir},
         vardir  => $params{vardir},
         targets => [],
-        tasks   => []
     };
     bless $self, $class;
 
@@ -58,6 +57,7 @@ sub init {
     # Skip create object if still defined (re-init case)
     my $config = $self->{config} || FusionInventory::Agent::Config->new(
         options => $params{options},
+        vardir  => $self->{vardir},
     );
     $self->{config} = $config;
 
@@ -91,7 +91,6 @@ sub init {
     my %available = $self->getAvailableTasks(disabledTasks => $config->{'no-task'});
     my @tasks = keys %available;
     my @plannedTasks = $self->computeTaskExecutionPlan(\@tasks);
-    $self->{tasksExecutionPlan} = \@plannedTasks;
 
     my %available_lc = map { (lc $_) => $_ } keys %available;
     if (!@tasks) {
@@ -103,13 +102,27 @@ sub init {
     foreach my $task (keys %available) {
         $logger->debug("- $task: $available{$task}");
     }
-    $logger->debug("Planned tasks:");
-    foreach my $task (@{$self->{tasksExecutionPlan}}) {
-        my $task_lc = lc $task;
-        $logger->debug("- $task: " . $available{$available_lc{$task_lc}});
-    }
 
-    $self->{tasks} = \@tasks;
+    foreach my $target ($self->getTargets()) {
+        if ($target->isType('local') || $target->isType('server')) {
+            $logger->debug("target $target->{id}: " . $target->getType() . " " . $target->getName());
+        } else {
+            $logger->debug("target $target->{id}: " . $target->getType());
+        }
+
+        # Register planned tasks by target
+        my @planned = $target->plannedTasks(@plannedTasks);
+
+        if (@planned) {
+            $logger->debug("Planned tasks for $target->{id}:");
+            foreach my $task (@planned) {
+                my $task_lc = lc $task;
+                $logger->debug("- $task: " . $available{$available_lc{$task_lc}});
+            }
+        } else {
+            $logger->debug("No planned task");
+        }
+    }
 
     $logger->info("Options 'no-task' and 'tasks' are both used. Be careful that 'no-task' always excludes tasks.")
         if ($self->{config}->isParamArrayAndFilled('no-task') && $self->{config}->isParamArrayAndFilled('tasks'));
@@ -172,16 +185,20 @@ sub terminate {
 sub runTarget {
     my ($self, $target) = @_;
 
+    if ($target->isType('local') || $target->isType('server')) {
+        $self->{logger}->info("target $target->{id}: " . $target->getType() . " " . $target->getName());
+    }
+
     # the prolog dialog must be done once for all tasks,
     # but only for server targets
     my $response;
-    if ($target->isa('FusionInventory::Agent::Target::Server')) {
+    if ($target->isType('server')) {
 
         return unless FusionInventory::Agent::HTTP::Client::OCS->require();
 
         my $client = FusionInventory::Agent::HTTP::Client::OCS->new(
             logger       => $self->{logger},
-            timeout      => $self->{timeout},
+            timeout      => $self->{config}->{timeout},
             user         => $self->{config}->{user},
             password     => $self->{config}->{password},
             proxy        => $self->{config}->{proxy},
@@ -197,7 +214,7 @@ sub runTarget {
             deviceid => $self->{deviceid},
         );
 
-        $self->{logger}->info("sending prolog request to server $target->{id}");
+        $self->{logger}->info("sending prolog request to $target->{id}");
         $response = $client->send(
             url     => $target->getUrl(),
             message => $prolog
@@ -215,7 +232,7 @@ sub runTarget {
         }
     }
 
-    foreach my $name (@{$self->{tasksExecutionPlan}}) {
+    foreach my $name ($target->plannedTasks()) {
         eval {
             $self->runTask($target, $name, $response);
         };
@@ -246,7 +263,11 @@ sub runTaskReal {
 
     my $class = "FusionInventory::Agent::Task::$name";
 
-    $class->require();
+    if (!$class->require()) {
+        $self->{logger}->debug2("$name task module does not compile: $@")
+            if $self->{logger};
+        return;
+    }
 
     my $task = $class->new(
         config       => $self->{config},
@@ -347,29 +368,6 @@ sub getAvailableTasks {
     return %tasks;
 }
 
-sub _getTaskVersion {
-    my ($self, $module) = @_;
-
-    my $logger = $self->{logger};
-
-    if (!$module->require()) {
-        $logger->debug2("module $module does not compile: $@") if $logger;
-
-        # Don't keep trace of module, only really needed to fix perl 5.8 issue
-        delete $INC{module2file($module)};
-
-        return;
-    }
-
-    my $version;
-    {
-        no strict 'refs';  ## no critic
-        $version = &{$module . '::VERSION'};
-    }
-
-    return $version;
-}
-
 sub _handlePersistentState {
     my ($self) = @_;
 
@@ -477,12 +475,6 @@ sub _makeExecutionPlan {
     return @executionPlan;
 }
 
-sub getTasksExecutionPlan {
-    my ($self) = @_;
-
-    return $self->{tasksExecutionPlan};
-}
-
 1;
 __END__
 
@@ -553,5 +545,8 @@ pairs:
 
 =head1 LICENSE
 
-This software is licensed under the terms of GPLv2+, see LICENSE file for
-details.
+This is free software, licensed under:
+
+  The GNU General Public License, Version 2, June 1991
+
+See LICENSE file for details.
