@@ -7,6 +7,7 @@ use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::SNMP;
 use FusionInventory::Agent::Tools::Network;
 use FusionInventory::Agent::SNMP::MibSupport;
+use FusionInventory::Agent::SNMP::Device::Components;
 
 # Supported infos are specified here:
 # http://fusioninventory.org/documentation/dev/spec/protocol/netdiscovery.html
@@ -16,9 +17,55 @@ use constant discovery => [ qw(
     )];
 # http://fusioninventory.org/documentation/dev/spec/protocol/netinventory.html
 use constant inventory => [ qw(
-        INFO PORTS MODEMS FIRMWARES SIMCARDS PAGECOUNTERS CARTRIDGES
+        INFO PORTS MODEMS FIRMWARES SIMCARDS PAGECOUNTERS CARTRIDGES COMPONENTS
     )];
 
+# common base variables
+my $base_variables = {
+    SNMPHOSTNAME => {
+        oid  => [
+            '.1.3.6.1.2.1.1.5.0',
+            '.1.3.6.1.4.1.2699.1.2.1.2.1.1.2.1', # PRINTER-PORT-MONITOR-MIB, ppmPrinterName
+        ],
+        type => 'string',
+    },
+    LOCATION     => {
+        oid  => '.1.3.6.1.2.1.1.6.0',
+        type => 'string',
+    },
+    CONTACT      => {
+        oid  => '.1.3.6.1.2.1.1.4.0',
+        type => 'string',
+    },
+    UPTIME       => {
+        oid  => '.1.3.6.1.2.1.1.3.0',
+        type => 'string',
+    },
+};
+
+# common base variables for inventory only
+my $inventory_only_base_variables = {
+    CPU          => {
+        oid  => [
+            '.1.3.6.1.4.1.9.9.109.1.1.1.1.6.1',
+            '.1.3.6.1.4.1.9.9.109.1.1.1.1.3.1',
+        ],
+        type => 'count',
+    },
+    MEMORY       => {
+        oid  => '.1.3.6.1.2.1.25.2.3.1.5.1',
+        type => 'memory',
+    },
+    RAM          => {
+        oid  => [
+            '.1.3.6.1.4.1.2021.4.5',
+            '.1.3.6.1.4.1.9.3.6.6.0',
+            '.1.3.6.1.2.1.25.2.2.0',
+        ],
+        type => 'memory',
+    },
+};
+
 sub new {
     my ($class, %params) = @_;
 
@@ -142,6 +189,14 @@ sub getInventory {
     return $inventory;
 }
 
+sub addComponent {
+    my ($self, $component) = @_;
+
+    return unless _cleanHash($component);
+
+    push @{$self->{COMPONENTS}->{COMPONENT}}, $component;
+}
+
 sub addModem {
     my ($self, $modem) = @_;
 
@@ -186,7 +241,7 @@ sub _cleanHash {
         $keys++;
         next if defined($hashref->{$key});
         delete $hashref->{$key};
-        $keys--,
+        $keys--;
     }
 
     return $keys;
@@ -196,12 +251,12 @@ sub setSerial {
     my ($self) = @_;
 
     my $serial =
+        # First try MIB Support mechanism
+        $self->getSerialByMibSupport()                         ||
         # Entity-MIB::entPhysicalSerialNum
         $self->{snmp}->get_first('.1.3.6.1.2.1.47.1.1.1.1.11') ||
         # Printer-MIB::prtGeneralSerialNumber
-        $self->{snmp}->get_first('.1.3.6.1.2.1.43.5.1.1.17') ||
-        # Try MIB Support mechanism
-        $self->getSerialByMibSupport();
+        $self->{snmp}->get_first('.1.3.6.1.2.1.43.5.1.1.17');
 
     if ( not defined $serial ) {
         # vendor specific OIDs
@@ -216,6 +271,9 @@ sub setSerial {
             '.1.3.6.1.4.1.318.1.1.4.1.5.0',          # MasterSwitch-MIB
             '.1.3.6.1.4.1.6027.3.8.1.1.5.0',         # F10-C-SERIES-CHASSIS-MIB
             '.1.3.6.1.4.1.6027.3.10.1.2.2.1.12.1',   # FORCE10-SMI
+            '.1.3.6.1.4.1.3417.2.11.1.4.0',          # BLUECOAT-SG-PROXY-MIB
+            '.1.3.6.1.4.1.232.2.2.2.1.0',            # CPQSINFO-MIB
+            '.1.3.6.1.4.1.232.11.2.10.3.0',          # CPQHOST-MIB
         );
         foreach my $oid (@oids) {
             $serial = $self->get($oid);
@@ -223,21 +281,26 @@ sub setSerial {
         }
     }
 
+    $serial = getCanonicalSerialNumber($serial);
+
     return unless $serial;
 
-    $self->{SERIAL} = getCanonicalSerialNumber($serial);
+    # Skip well-known invalid serial number
+    return if $serial =~ /^X+$/;
+
+    $self->{SERIAL} = $serial;
 }
 
 sub setFirmware {
     my ($self) = @_;
 
     my $firmware =
+        # First try to get firmware from MIB Support mechanism
+        $self->getFirmwareByMibSupport()                       ||
         # entPhysicalSoftwareRev
         $self->{snmp}->get_first('.1.3.6.1.2.1.47.1.1.1.1.10') ||
         # entPhysicalFirmwareRev
-        $self->{snmp}->get_first('.1.3.6.1.2.1.47.1.1.1.1.9')  ||
-        # firmware from supported mib
-        $self->getFirmwareByMibSupport();
+        $self->{snmp}->get_first('.1.3.6.1.2.1.47.1.1.1.1.9');
 
     if ( not defined $firmware ) {
         # vendor specific OIDs
@@ -250,12 +313,15 @@ sub setFirmware {
             $firmware = $self->get($oid);
             last if defined $firmware;
         }
+        return unless defined $firmware;
     }
 
-    return unless defined $firmware;
+    $firmware = getCanonicalString($firmware);
+
+    return unless $firmware;
 
     # Set device firmware
-    $self->{FIRMWARE} = getCanonicalString($firmware);
+    $self->{FIRMWARE} = $firmware;
 
     # Also add firmware as device FIRMWARES
     $self->addFirmware({
@@ -333,6 +399,112 @@ sub setMacAddress {
     }
 }
 
+# rules on model name to reset manufacturer to real vendor
+my %sysmodel_first_word = (
+    'dell'           => { manufacturer => 'Dell', },
+);
+
+sub setModel {
+    my ($self) = @_;
+
+    # fallback model identification attempt, using type-specific OID value
+    if (!exists $self->{MODEL}) {
+        my $model = exists $self->{TYPE} && $self->{TYPE} eq 'PRINTER' ?
+            $self->get('.1.3.6.1.2.1.25.3.2.1.3.1')    :
+            exists $self->{TYPE} && $self->{TYPE} eq 'POWER' ?
+            $self->get('.1.3.6.1.2.1.33.1.1.5.0')      : # UPS-MIB
+            $self->get('.1.3.6.1.2.1.47.1.1.1.1.13.1') ;
+        $self->{MODEL} = getCanonicalString($model) if $model;
+    }
+
+    # fallback manufacturer identification attempt, using type-agnostic OID
+    if (!exists $self->{MANUFACTURER}) {
+        my $manufacturer = $self->get('.1.3.6.1.2.1.43.8.2.1.14.1.1');
+        $self->{MANUFACTURER} = $manufacturer if $manufacturer;
+    }
+
+    # reset manufacturer by rule as real vendor based on first model word
+    if (exists $self->{MODEL}) {
+        my ($first_word) = $self->{MODEL} =~ /(\S+)/;
+        my $result = $sysmodel_first_word{lc($first_word)};
+        if ($result && $result->{manufacturer}) {
+            $self->{MANUFACTURER} = $result->{manufacturer};
+        }
+    }
+
+    # Permit mib support to reset model
+    if ($self->{MIBSUPPORT}) {
+        my $model = $self->{MIBSUPPORT}->getMethod('getModel');
+        $self->{MODEL} = getCanonicalString($model) if $model;
+    }
+}
+
+sub setType {
+    my ($self) = @_;
+
+    # Permit mib support to reset type
+    if ($self->{MIBSUPPORT}) {
+        my $type = $self->{MIBSUPPORT}->getMethod('getType');
+        $self->{TYPE} = $type if $type;
+    }
+}
+
+sub setManufacturer {
+    my ($self) = @_;
+
+    # Permit mib support to reset manufacturer
+    if ($self->{MIBSUPPORT}) {
+        my $manufacturer = $self->{MIBSUPPORT}->getMethod('getManufacturer');
+        $self->{MANUFACTURER} = $manufacturer if $manufacturer;
+    }
+}
+
+sub setBaseInfos {
+    my ($self) = @_;
+    $self->_set_from_oid_list($base_variables, $self);
+
+    # Filter out LOCATION & CONTACT from unwanted values
+    if ($self->{LOCATION} && $self->{LOCATION} =~ /edit \/etc.*snmp.*\.conf/) {
+        delete $self->{LOCATION};
+    }
+    if ($self->{CONTACT} && $self->{CONTACT} =~ /configure \/etc.*snmp.*\.conf/) {
+        delete $self->{CONTACT};
+    }
+}
+
+sub setInventoryBaseInfos {
+    my ($self) = @_;
+    $self->_set_from_oid_list($inventory_only_base_variables, $self->{INFO});
+}
+
+sub _set_from_oid_list {
+    my ($self, $list, $where) = @_;
+
+    foreach my $key (keys %{$list}) {
+        my $variable = $list->{$key};
+
+        my $raw_value;
+        if (ref $variable->{oid} eq 'ARRAY') {
+            foreach my $oid (@{$variable->{oid}}) {
+                $raw_value = $self->get($oid);
+                last if defined $raw_value;
+            }
+        } else {
+            $raw_value = $self->get($variable->{oid});
+        }
+        next unless defined $raw_value;
+
+        my $type = $variable->{type};
+        my $value =
+            $type eq 'memory' ? getCanonicalMemory($raw_value) :
+            $type eq 'string' ? getCanonicalString($raw_value) :
+            $type eq 'count'  ? getCanonicalCount($raw_value)  :
+                                $raw_value;
+
+        $where->{$key} = $value if defined $value;
+    }
+}
+
 sub _numericMac {
     my ($mac) = @_;
 
@@ -361,6 +533,28 @@ sub setIp {
     $self->{IPS}->{IP} = [ $ip ] if $ip;
 }
 
+sub setComponents {
+    my ($self) = @_;
+
+    # First try to get components from standard ENTITY-MIB support
+    my $components = FusionInventory::Agent::SNMP::Device::Components->new(
+        device => $self
+    );
+    if ($components) {
+        foreach my $component (@{$components->getPhysicalComponents()}) {
+            $self->addComponent($component);
+        }
+    }
+
+    # Then look for any plugin providing getComponents API
+    if ($self->{MIBSUPPORT}) {
+        $components = $self->{MIBSUPPORT}->getMethod('getComponents') || [];
+        foreach my $component (@{$components}) {
+            $self->addComponent($component);
+        }
+    }
+}
+
 1;
 
 __END__
