@@ -11,6 +11,10 @@ use FusionInventory::Agent::Tools::SNMP;
 use constant    apc => '.1.3.6.1.4.1.318' ;
 use constant    riello => '.1.3.6.1.4.1.5491' ;
 
+# See PowerNet-MIB
+
+use constant    upsAdvIdentSerialNumber => apc . '.1.1.1.1.2.3.0';
+
 # See UPS-MIB
 
 use constant    upsMIB  => '.1.3.6.1.2.1.33' ;
@@ -33,6 +37,12 @@ sub getModel {
     return $self->get(upsIdentModel);
 }
 
+sub getSerial {
+    my ($self) = @_;
+
+    return $self->get(upsAdvIdentSerialNumber);
+}
+
 sub getFirmware {
     my ($self) = @_;
 
@@ -56,7 +66,7 @@ __END__
 
 =head1 NAME
 
-Inventory module for APC modules
+FusionInventory::Agent::SNMP::MibSupport::UPS - Inventory module for APC modules
 
 =head1 DESCRIPTION
 
