@@ -6,7 +6,7 @@ use warnings;
 use English qw(-no_match_vars);
 use UNIVERSAL::require;
 
-use base "FusionInventory::Agent::Task::Deploy::CheckProcessor";
+use parent "FusionInventory::Agent::Task::Deploy::CheckProcessor";
 
 sub prepare {
     my ($self) = @_;
