@@ -6,7 +6,6 @@ use warnings;
 use parent 'FusionInventory::Agent::Task::Inventory::Module';
 
 use English qw(-no_match_vars);
-use File::Basename qw(basename);
 
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Generic;
@@ -22,11 +21,10 @@ sub isEnabled {
 sub doInventory {
     my (%params) = @_;
 
-    my $inventory = delete $params{inventory};
+    my $inventory = $params{inventory};
+    my $logger    = $params{logger};
 
-    $params{root} = $params{test_path} || "";
-
-    foreach my $device (_getDevices(%params)) {
+    foreach my $device (_getDevices(logger => $logger)) {
         $inventory->addEntry(section => 'STORAGES', entry => $device);
     }
 }
@@ -34,15 +32,15 @@ sub doInventory {
 sub _getDevices {
     my (%params) = @_;
 
-    my $root   = $params{root};
+    my $logger    = $params{logger};
 
-    my @devices = _getDevicesBase(%params);
+    my @devices = _getDevicesBase(logger => $logger);
 
     # complete with udev for missing bits, if available
-    if (-d "$root/dev/.udev/db/") {
+    if (-d '/dev/.udev/db/') {
 
         my %udev_devices = map { $_->{NAME} => $_ }
-            getDevicesFromUdev(%params);
+            getDevicesFromUdev(logger => $logger);
 
         foreach my $device (@devices) {
             # find corresponding udev entry
@@ -57,12 +55,12 @@ sub _getDevices {
     }
 
     # get serial & firmware numbers from hdparm, if available
-    if (_correctHdparmAvailable(%params)) {
+    if (_correctHdparmAvailable()) {
         foreach my $device (@devices) {
             next if $device->{SERIALNUMBER} && $device->{FIRMWARE};
             my $info = getHdparmInfo(
                 device => "/dev/" . $device->{NAME},
-                %params
+                logger  => $logger
             );
 
             $device->{SERIALNUMBER} = $info->{serial}
@@ -96,16 +94,12 @@ sub _getDevices {
         }
 
         if (!$device->{DISKSIZE} && $device->{TYPE} !~ /^cd/) {
-            $device->{DISKSIZE} = getDeviceCapacity(
-                device => '/dev/' . $device->{NAME},
-                %params
-            );
+            $device->{DISKSIZE} = getDeviceCapacity(device => '/dev/' . $device->{NAME});
         }
 
         # In some case, serial can't be defined using hdparm (command missing or virtual disk)
         # Then we can define a serial searching for few specific identifiers
-        # But avoid to search S/N for empty removable meaning no disk has been inserted
-        if (!$device->{SERIALNUMBER} && !($device->{TYPE} eq 'removable' && !$device->{DISKSIZE})) {
+        if (!$device->{SERIALNUMBER}) {
             $params{device} = '/dev/' . $device->{NAME};
             my $sn = _getDiskIdentifier(%params) || _getPVUUID(%params);
             $device->{SERIALNUMBER} = $sn if $sn;
@@ -118,13 +112,11 @@ sub _getDevices {
 sub _getDevicesBase {
     my (%params) = @_;
 
-    # We need to support dump params to permit full testing when root params is set
-    my $root   = $params{root};
     my $logger = $params{logger};
     $logger->debug("retrieving devices list:");
 
-    if (-d "$root/sys/block") {
-        my @devices = getDevicesFromProc(%params);
+    if (-d '/sys/block') {
+        my @devices = getDevicesFromProc(logger => $logger);
         $logger->debug_result(
             action => 'reading /sys/block content',
             data   => scalar @devices
@@ -137,8 +129,8 @@ sub _getDevicesBase {
         );
     }
 
-    if ((!$root && canRun("/usr/bin/lshal")) || ($root && -e "$root/lshal")) {
-        my @devices = getDevicesFromHal(%params);
+    if (canRun('/usr/bin/lshal')) {
+        my @devices = getDevicesFromHal(logger => $logger);
         $logger->debug_result(
             action => 'running lshal command',
             data   => scalar @devices
@@ -171,8 +163,6 @@ sub _fixDescription {
         } else {
             return "SCSI";
         }
-    } elsif ($name =~ /^sg/) { # "g" stands for Generic SCSI
-        return "SCSI";
     } elsif ($name =~ /^vd/  ||
             ($description && $description =~ /VIRTIO/)
         ) {
@@ -186,25 +176,11 @@ sub _fixDescription {
 # run on CDROM device
 # http://forums.ocsinventory-ng.org/viewtopic.php?pid=20810
 sub _correctHdparmAvailable {
-    my (%params) = @_;
-
-    $params{command} = "hdparm -V";
-
-    # We need to support dump params to permit full testing when root params is set
-    if ($params{root}) {
-        $params{file} = $params{root}."/hdparm";
-        return unless -e $params{file};
-    }
-
-    return unless canRun('hdparm') || $params{file};
-
-    if ($params{dump}) {
-        $params{dump}->{"hdparm"} = getAllLines(%params);
-    }
+    return unless canRun('hdparm');
 
     my ($major, $minor) = getFirstMatch(
-        pattern => qr/^hdparm v(\d+)\.(\d+)/,
-        %params
+        command => 'hdparm -V',
+        pattern => qr/^hdparm v(\d+)\.(\d+)/
     );
 
     # we need at least version 9.15
@@ -215,37 +191,17 @@ sub _correctHdparmAvailable {
 sub _getDiskIdentifier {
     my (%params) = @_;
 
-    # We need to support dump params to permit full testing when root params is set
-    if ($params{root}) {
-        $params{file} = $params{root}."/fdisk";
-        return unless -e $params{file};
-    } else {
-        $params{command} = "fdisk -v";
-    }
-
-    return unless $params{device} && (canRun("fdisk") || $params{file});
-
-    if ($params{dump}) {
-        $params{dump}->{"fdisk"} = getAllLines(%params);
-    }
+    return unless $params{device} && canRun("fdisk");
 
     # GNU version requires -p flag
-    $params{command} = getFirstLine(%params) =~ '^GNU' ?
+    my $command = getFirstLine(command => 'fdisk -v') =~ '^GNU' ?
         "fdisk -p -l $params{device}" :
         "fdisk -l $params{device}"    ;
 
-    if ($params{root}) {
-        my $devname = basename($params{device});
-        $params{file} = $params{root}."/fdisk-$devname";
-        return unless -e $params{file};
-    } elsif ($params{dump}) {
-        my $devname = basename($params{device});
-        $params{dump}->{"fdisk-$devname"} = getAllLines(%params);
-    }
-
     my $identifier = getFirstMatch(
+        command => $command,
         pattern => qr/^Disk identifier:\s*(?:0x)?(\S+)$/i,
-        %params
+        logger  => $params{logger},
     );
 
     return $identifier;
@@ -254,25 +210,14 @@ sub _getDiskIdentifier {
 sub _getPVUUID {
     my (%params) = @_;
 
-    # We need to support dump params to permit full testing when root params is set
-    if ($params{root}) {
-        my $devname = basename($params{device});
-        $params{file} = $params{root}."/lvm-$devname";
-        return unless -e $params{file};
-    }
+    return unless $params{device} && canRun("lvm");
 
-    return unless $params{device} && (canRun("lvm") || $params{file});
-
-    $params{command} = "lvm pvdisplay -C -o pv_uuid --noheadings $params{device}" ;
-
-    if ($params{dump}) {
-        my $devname = basename($params{device});
-        $params{dump}->{"lvm-$devname"} = getAllLines(%params);
-    }
+    my $command = "lvm pvdisplay -C -o pv_uuid --noheadings $params{device}" ;
 
     my $uuid = getFirstMatch(
+        command => $command,
         pattern => qr/^\s*(\S+)/,
-        %params
+        logger  => $params{logger},
     );
 
     return $uuid;
