@@ -101,7 +101,13 @@ sub _getSerialATAStorages {
         my $storage = _extractStorage($hash);
         $storage->{TYPE} = 'Disk drive';
         $storage->{INTERFACE} = 'SERIAL-ATA';
-        push @storages, _sanitizedHash($storage);
+        %$storage = map {
+            my $value = $storage->{$_};
+            $value =~ s/^\s*//g;
+            $value =~ s/\s*$//g;
+            $_ => $value
+        } keys %$storage;
+        push @storages, $storage;
     }
 
     return @storages;
@@ -143,7 +149,13 @@ sub _getDiscBurningStorages {
     for my $hash (values %{$infos->{storages}}) {
         my $storage = _extractDiscBurning($hash);
         $storage->{TYPE} = 'Disk burning';
-        push @storages, _sanitizedHash($storage);
+        %$storage = map {
+            my $value = $storage->{$_};
+            $value =~ s/^\s*//g;
+            $value =~ s/\s*$//g;
+            $_ => $value
+        } keys %$storage;
+        push @storages, $storage;
     }
 
     return @storages;
@@ -188,7 +200,13 @@ sub _getCardReaderStorages {
             $storage = _extractSdCard($hash);
             $storage->{TYPE} = 'SD Card';
         }
-        push @storages, _sanitizedHash($storage);
+        %$storage = map {
+            my $value = $storage->{$_};
+            $value =~ s/^\s*//g;
+            $value =~ s/\s*$//g;
+            $_ => $value
+        } keys %$storage;
+        push @storages, $storage;
     }
 
     return @storages;
@@ -288,7 +306,13 @@ sub _getUSBStorages {
         my $storage = _extractUSBStorage($hash);
         $storage->{TYPE} = 'Disk drive';
         $storage->{INTERFACE} = 'USB';
-        push @storages, _sanitizedHash($storage);
+        %$storage = map {
+            my $value = $storage->{$_} || '';
+            $value =~ s/^\s*//g;
+            $value =~ s/\s*$//g;
+            $_ => $value
+        } keys %$storage;
+        push @storages, $storage;
     }
 
     return @storages;
@@ -318,6 +342,24 @@ sub _extractDiskSize {
             getCanonicalSize($hash->{size}, 1024);
 }
 
+sub _cleanSizeString {
+    my ($sizeString) = @_;
+
+    return unless $sizeString =~ /^(\d+(?:(?:\.|,)\d+)?) /;
+    my $nbStr = $1;
+    $nbStr =~ s/,/./;
+
+    return $nbStr;
+}
+
+sub _getSizeUnit {
+    my ($sizeString) = @_;
+
+    return 'GB' if $sizeString =~ /GB$/;
+    return 'MB' if $sizeString =~ /MB$/;
+    return '';
+}
+
 sub _extractValueInHashWithKeyPattern {
     my ($pattern, $hash) = @_;
 
@@ -345,7 +387,13 @@ sub _getFireWireStorages {
         my $storage = _extractFireWireStorage($hash);
         $storage->{TYPE} = 'Disk drive';
         $storage->{INTERFACE} = 'FireWire';
-        push @storages, _sanitizedHash($storage);
+        %$storage = map {
+            my $value = $storage->{$_} || '';
+            $value =~ s/^\s*//g;
+            $value =~ s/\s*$//g;
+            $_ => $value
+        } keys %$storage;
+        push @storages, $storage;
     }
 
     return @storages;
@@ -367,16 +415,4 @@ sub _extractFireWireStorage {
     return $storage;
 }
 
-sub _sanitizedHash {
-    my ($hash) = @_;
-    foreach my $key (keys(%{$hash})) {
-        if (defined($hash->{$key})) {
-            $hash->{$key} = trimWhitespace($hash->{$key});
-        } else {
-            delete $hash->{$key};
-        }
-    }
-    return $hash;
-}
-
 1;
