@@ -7,6 +7,7 @@ use parent 'FusionInventory::Agent::Task::Inventory::Module';
 
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Network;
+use FusionInventory::Agent::Tools::Virtualization;
 
 sub isEnabled {
     # Avoid duplicated entry with libvirt
@@ -18,28 +19,54 @@ sub isEnabled {
 sub doInventory {
     my (%params) = @_;
 
+    my $inventory = $params{inventory};
+
+    foreach my $vz (_parseVzlist(%params)) {
+        $inventory->addEntry(
+            section => 'VIRTUALMACHINES',
+            entry => $vz
+        );
+    }
+}
+
+sub _parseVzlist {
+    my (%params) = @_;
+
     my $inventory = $params{inventory};
     my $logger    = $params{logger};
 
     my $handle = getFileHandle(
         command => 'vzlist --all --no-header -o hostname,ctid,cpulimit,status,ostemplate',
-        logger  => $logger
+        %params
     );
 
     return unless $handle;
 
-    # no service containers in glpi
-    my $line = <$handle>;
+    my $vzlist;
+    my $confctid_template = $params{ctid_template} ||
+        "/etc/vz/conf/__XXX__.conf";
 
     my $hostID = $inventory->getHardware('UUID') || '';
 
+    my %status_list = (
+        'stopped'   => STATUS_OFF,
+        'running'   => STATUS_RUNNING,
+        'paused'    => STATUS_PAUSED,
+        'mounted'   => STATUS_OFF,
+        'suspended' => STATUS_PAUSED,
+        'unknown'   => STATUS_OFF,
+    );
+
     while (my $line = <$handle>) {
 
         chomp $line;
         my ($name, $ctid, $cpus, $status, $subsys) = split(/[ \t]+/, $line);
 
+        my $ctid_conf = $confctid_template;
+        $ctid_conf =~ s/__XXX__/$ctid/;
+
         my $memory = getFirstMatch(
-            file    => "/etc/vz/conf/$ctid.conf",
+            file    => $ctid_conf,
             pattern => qr/^SLMMEMORYLIMIT="\d+:(\d+)"$/,
             logger  => $logger,
         );
@@ -47,7 +74,7 @@ sub doInventory {
             $memory = $memory / 1024 / 1024;
         } else {
             $memory = getFirstMatch(
-                file    => "/etc/vz/conf/$ctid.conf",
+                file    => $ctid_conf,
                 pattern => qr/^PRIVVMPAGES="\d+:(\d+)"$/,
                 logger  => $logger,
             );
@@ -55,7 +82,7 @@ sub doInventory {
                 $memory = $memory * 4 / 1024;
             } else {
                 $memory = getFirstMatch(
-                    file    => "/etc/vz/conf/$ctid.conf",
+                    file    => $ctid_conf,
                     pattern => qr/^PHYSPAGES="\d+:(\d+\w{0,1})"$/,
                     logger  => $logger,
                 );
@@ -78,31 +105,42 @@ sub doInventory {
         # unique only for the local hosts
         my $uuid = $hostID . '-' . $ctid;
 
-        $inventory->addEntry(
-            section => 'VIRTUALMACHINES',
-            entry => {
-                NAME      => $name,
-                VCPU      => $cpus,
-                UUID      => $uuid,
-                MEMORY    => $memory,
-                STATUS    => $status,
-                SUBSYSTEM => $subsys,
-                VMTYPE    => "Virtuozzo",
-                MAC       => _getMACs($ctid, $logger)
-            }
-        );
+        $status = $status_list{$status} || STATUS_OFF;
 
+        my $container = {
+            NAME      => $name,
+            VCPU      => $cpus,
+            UUID      => $uuid,
+            MEMORY    => $memory,
+            STATUS    => $status,
+            SUBSYSTEM => $subsys,
+            VMTYPE    => "Virtuozzo",
+        };
+
+        my $mac = _getMACs(
+            status  => $status,
+            ctid    => $ctid,
+            logger  => $logger
+        );
+        $container->{MAC} = $mac if $mac;
+
+        push @{$vzlist}, $container;
     }
 
     close $handle;
+
+    return $vzlist;
 }
 
 sub _getMACs {
-    my ($ctid, $logger) = @_;
+    my (%params) = @_;
+
+    # Don't try to run a command in a not running container
+    return unless $params{status} && $params{status} eq STATUS_RUNNING;
 
     my @ipLines = getAllLines(
-        command => "vzctl exec '$ctid' 'ip -0 a'",
-        logger  => $logger
+        command => "vzctl exec '$params{ctid}' 'ip -0 a'",
+        %params
     );
 
     my @macs;
