@@ -29,7 +29,7 @@ our $VERSION = FusionInventory::Agent::Task::NetInventory::Version::VERSION;
 sub isEnabled {
     my ($self, $response) = @_;
 
-    if (!$self->{target}->isa('FusionInventory::Agent::Target::Server')) {
+    if (!$self->{target}->isType('server')) {
         $self->{logger}->debug("NetInventory task not compatible with local target");
         return;
     }
@@ -94,7 +94,6 @@ sub run {
     ) if !$self->{client};
 
     foreach my $job (@{$self->{jobs}}) {
-        my $pid         = $job->{params}->{PID};
         my $max_threads = $job->{params}->{THREADS_QUERY};
         my $timeout     = $job->{params}->{TIMEOUT};
 
@@ -102,10 +101,13 @@ sub run {
         my $credentials = _getIndexedCredentials($job->{credentials});
 
         # set internal state
-        $self->{pid} = $pid;
+        $self->{pid} = $job->{params}->{PID} || '';
 
-        # send initial message to the server
-        $self->_sendStartMessage();
+        # newer server won't need START message if PID is provided on <DEVICE/>
+        my $skip_start_stop = any { defined($_->{PID}) } @{$job->{devices}};
+
+        # send initial message to server unless it supports newer protocol
+        $self->_sendStartMessage() unless $skip_start_stop;
 
         my ($debug_sent_count, $started_count) = ( 0, 0 );
         my %running_threads = ();
@@ -148,10 +150,15 @@ sub run {
                     $result = {
                         ERROR => {
                             ID      => $device->{ID},
-                            TYPE    => $device->{TYPE},
                             MESSAGE => $EVAL_ERROR
                         }
                     };
+
+                    $result->{ERROR}->{TYPE} = $device->{TYPE} if $device->{TYPE};
+
+                    # Inserted back device PID in result if set by server
+                    $result->{PID} = $device->{PID} if defined($device->{PID});
+
                     $self->{logger}->error("[thread $id] $EVAL_ERROR");
                 }
 
@@ -187,7 +194,7 @@ sub run {
             }
 
             if ($expiration && time > $expiration) {
-                $self->{logger}->warn("Aborting netinventory job as it reached expiration time");
+                $self->{logger}->warning("Aborting netinventory job as it reached expiration time");
                 # detach all our running worker
                 foreach my $tid (keys(%running_threads)) {
                     $running_threads{$tid}->detach()
@@ -228,16 +235,16 @@ sub run {
             $self->_sendResultMessage($result);
         }
 
-        # send final message to the server before cleaning threads
-        $self->_sendStopMessage();
+        # send final message to the server before cleaning threads unless it supports newer protocol
+        $self->_sendStopMessage() unless $skip_start_stop;
 
         if ($started_count) {
             $self->{logger}->debug("cleaning $started_count worker threads");
             $_->join() foreach threads->list(threads::joinable);
         }
 
-        # send final message to the server
-        $self->_sendStopMessage();
+        # send final message to the server unless it supports newer protocol
+        $self->_sendStopMessage() unless $skip_start_stop;
     }
 }
 
@@ -285,10 +292,17 @@ sub _sendStopMessage {
 sub _sendResultMessage {
     my ($self, $result) = @_;
 
+    my $pid = $self->{pid};
+    if (exists($result->{PID})) {
+        # Don't keep PID in result but we want to set it as parent PROCESSNUMBER
+        $pid = $result->{PID} if defined($result->{PID});
+        delete $result->{PID};
+    }
+
     $self->_sendMessage({
         DEVICE        => $result,
         MODULEVERSION => $VERSION,
-        PROCESSNUMBER => $self->{pid}
+        PROCESSNUMBER => $pid
     });
 }
 
@@ -299,7 +313,11 @@ sub _queryDevice {
     my $device      = $params{device};
     my $logger      = $self->{logger};
     my $id          = threads->tid();
-    $logger->debug("[thread $id] scanning $device->{ID}");
+    $logger->debug(
+        "[thread $id] scanning $device->{ID}: $device->{IP}" .
+        ( $device->{PORT} ? ' on port ' . $device->{PORT} : '' ) .
+        ( $device->{PROTOCOL} ? ' via ' . $device->{PROTOCOL} : '' )
+    );
 
     my $snmp;
     if ($device->{FILE}) {
@@ -317,6 +335,8 @@ sub _queryDevice {
             $snmp = FusionInventory::Agent::SNMP::Live->new(
                 version      => $credentials->{VERSION},
                 hostname     => $device->{IP},
+                port         => $device->{PORT},
+                domain       => $device->{PROTOCOL},
                 timeout      => $params{timeout} || 15,
                 community    => $credentials->{COMMUNITY},
                 username     => $credentials->{USERNAME},
@@ -338,6 +358,9 @@ sub _queryDevice {
          datadir => $self->{datadir}
     );
 
+    # Inserted back device PID in result if set by server
+    $result->{PID} = $device->{PID} if defined($device->{PID});
+
     return $result;
 }
 
