@@ -250,6 +250,16 @@ sub getRegistryValue {
         return _call_win32_ole_dependent_api($win32_ole_dependent_api);
     }
 
+    # Handle differently paths including /**/ pattern
+    if ($root =~ m/\/\*\*(?:\/.*|)$/ || $keyName eq '**') {
+        return _getRegistryDynamic(
+            logger    => $params{logger},
+            path      => "$root/$keyName",
+            valueName => $valueName,
+            withtype  => $params{withtype}
+        );
+    }
+
     my $key = _getRegistryKey(
         logger  => $params{logger},
         root    => $root,
@@ -260,7 +270,7 @@ sub getRegistryValue {
 
     if ($valueName eq '*') {
         my %ret;
-        foreach (keys %$key) {
+        foreach (grep { m|^/| } keys %$key) {
             s{^/}{};
             $ret{$_} = $params{withtype} ? [$key->GetValue($_)] : $key->{"/$_"} ;
         }
@@ -367,7 +377,7 @@ sub getRegistryKey {
     );
 }
 
-sub _getRegistryKey {
+sub _getRegistryRoot {
     my (%params) = @_;
 
     ## no critic (ProhibitBitwise)
@@ -381,11 +391,72 @@ sub _getRegistryKey {
         ) if $params{logger};
         return;
     }
+    return $rootKey;
+}
+
+sub _getRegistryKey {
+    my (%params) = @_;
+
+    my $rootKey = _getRegistryRoot(%params)
+        or return;
+
     my $key = $rootKey->Open($params{keyName});
 
     return $key;
 }
 
+sub _getRegistryDynamic {
+    my (%params) = @_;
+
+    my %ret;
+    my $valueName = $params{valueName};
+
+    my @rootparts = split(/\/+\*\*\/+/, $params{path}.'/', 2);
+    my $first = shift(@rootparts);
+    my $second = shift(@rootparts) || '';
+    $first .= '/';
+    $second = '/'.$second;
+    $second =~ s|/*$||;
+
+    my $rootSub = _getRegistryRoot(
+        root    => $first,
+        logger  => $params{logger}
+    );
+    return unless defined($rootSub);
+
+    foreach my $sub ($rootSub->SubKeyNames) {
+        if ($second =~ m/\/+\*\*(?:\/.*|)/) {
+            my $subret = _getRegistryDynamic(
+                logger    => $params{logger},
+                path      => $first.$sub.$second,
+                valueName => $valueName,
+                withtype  => $params{withtype}
+            );
+            next unless defined($subret);
+            my ($subkey) = $second =~ /^([^*]+)\*\*(?:\/.*|)$/;
+            foreach my $subretkey (keys %$subret) {
+                $ret{$sub.$subkey.$subretkey} = $subret->{$subretkey};
+            }
+        } else {
+            my $key = _getRegistryRoot(
+                root    => $first.$sub.$second,
+                logger  => $params{logger}
+            );
+            next unless defined($key);
+
+            if ($valueName eq '*') {
+                foreach (grep { m|^/| } keys %$key) {
+                    s{^/}{};
+                    $ret{$sub.$second."/".$_} = $params{withtype} ? [$key->GetValue($_)] : $key->{"/$_"} ;
+                }
+            } elsif (exists($key->{"/$valueName"})) {
+                $ret{$sub.$second."/".$valueName} = $params{withtype} ? [$key->GetValue($valueName)] : $key->{"/$valueName"} ;
+            }
+        }
+    }
+    return \%ret;
+}
+
 sub _getRegistryKeyFromWMI{
     my (%params) = @_;
 
@@ -674,7 +745,7 @@ sub getInterfaces {
                 $interface->{DNSDomain} = $configuration->{DNSDomain}
                     if $configuration->{DNSDomain};
 
-                $interface->{SPEED}      = $object->{Speed} / 1_000_000
+                $interface->{SPEED}      = int($object->{Speed} / 1_000_000)
                     if $object->{Speed};
                 $interface->{VIRTUALDEV} = _isVirtual($object, $configuration);
 
@@ -698,7 +769,7 @@ sub getInterfaces {
             $interface->{DNSDomain} = $configuration->{DNSDomain}
                 if $configuration->{DNSDomain};
 
-            $interface->{SPEED}      = $object->{Speed} / 1_000_000
+            $interface->{SPEED}      = int($object->{Speed} / 1_000_000)
                 if $object->{Speed};
             $interface->{VIRTUALDEV} = _isVirtual($object, $configuration);
 
@@ -801,7 +872,7 @@ sub getAgentMemorySize {
     };
     return -1 unless (defined($ph));
 
-    my $size = -1;
+    my ($size, $pages) = ( -1, 0 );
     eval {
         # memory usage is bundled up in ProcessMemoryCounters structure
         # populated by GetProcessMemoryInfo() win32 call
@@ -836,12 +907,24 @@ sub getAgentMemorySize {
             )'
         );
         if ($apiGetProcessMemoryInfo->Call($ph, $mem_counters, $cb)) {
-            # Uses WorkingSetSize as process memory size
+            # Uses WorkingSetSize and PagefileUsage
             $size = $mem_counters->{WorkingSetSize};
+            $pages = $mem_counters->{PagefileUsage};
         }
     };
 
-    return $size;
+    # Don't forget to close Process Handle
+    eval {
+        my $apiCloseHandle = Win32::API->new(
+            'kernel32',
+            'CloseHandle',
+            'I',
+            'I'
+        );
+        $ph = $apiCloseHandle->Call($ph);
+    };
+
+    return $size, $pages;
 }
 
 sub FreeAgentMem {
@@ -881,6 +964,10 @@ my @win32_ole_calls : shared;
 sub start_Win32_OLE_Worker {
 
     unless (defined($worker)) {
+
+        # Handle thread KILL signal
+        $SIG{KILL} = sub { threads->exit(); };
+
         # Request a semaphore on which worker blocks immediatly
         Thread::Semaphore->require();
         $worker_semaphore = Thread::Semaphore->new(0);
@@ -888,6 +975,8 @@ sub start_Win32_OLE_Worker {
         # Start a worker thread
         $worker = threads->create( \&_win32_ole_worker );
     }
+
+    return $worker;
 }
 
 sub setupWorkerLogger {
